# Import libraries
import requests
import pandas as pd
import numpy as np
import html5lib
from nba_data_miner import search
from nba_data_miner.helper_functions import abrv_translator
from nba_data_miner.helper_functions import get_espn_team,get_espn_player
from pandas import json_normalize

def teams(team1,team2):
    """
    Compares the stats of two NBA teams.

    Parameters
    ----------
    team1 : String
    team2 : String

    Returns
    -------
    A data frame of side-by-side stats for the two teams.
    Type: pandas.core.frame.DataFrame

    Examples
    --------
    >>> from nba_data_miner import compare
    >>> compare.teams("Suns","Nets")
          stat  Phoenix Suns  Brooklyn Nets
    0       GP         28.00          29.00
    1       GS           NaN            NaN
    2      MIN           NaN            NaN
    3      PTS        115.40         112.50
    4       OR         12.10           8.10
    5       DR         31.00          32.70
    6      REB         43.20          40.80
    7      AST         27.40          26.10
    8      STL          7.30           7.00
    9      BLK          5.50           7.00
    10      TO         13.00          14.10
    11      PF         21.80          21.90
    12  AST/TO          2.10           1.80
    13     FGM         43.10          41.90
    14     FGA         91.60          84.00
    15     FG%         47.00          49.90
    16     3PM         13.00          11.60
    17     3PA         34.50          31.40
    18     3P%         37.60          37.00
    19     FTM         16.30          17.00
    20     FTA         20.20          21.40
    21     FT%         80.70          79.40
    22     2PM         30.10          30.30
    23     2PA         57.00          52.60
    24     2P%         52.70          57.60
    25  SC-EFF          1.26           1.34
    26  SH-EFF          0.54           0.57
    
    * Values will be different when used but the returned
      data frame should match the above structure.
    """
    # Get names for web scraping
    abrv1 = search.team(team1).abrv
    abrv2 = search.team(team2).abrv
    
    # Scrape stats on both teams
    # Team 1
    stats1_r = requests.get('https://www.espn.com/nba/team/stats/_/name/' + 
                                            abrv_translator(abrv1)).content
    team1_dict = get_espn_team(stats1_r)
    
    # Team 2 
    stats2_r = requests.get('https://www.espn.com/nba/team/stats/_/name/' + 
                                            abrv_translator(abrv2)).content
    team2_dict = get_espn_team(stats2_r)

    # Return final data frame
    df_final = pd.DataFrame({'stat' : team2_dict['metrics'],
                             search.team(team1).name : np.round(team1_dict['values'],2),
                             search.team(team2).name : team2_dict['values']})

    return df_final
   


def players(player1,player2):
    """
    Compares the stats of two NBA players.

    Parameters
    ----------
    player1 : String
    player2 : String

    Returns
    -------
    A data frame of side-by-side stats for the two players.
    Type: pandas.core.frame.DataFrame

    Examples
    --------
    >>> from nba_data_miner import compare
    >>> compare.players("Al Horford","Lebron James")
          stat  Al Horford  LeBron James
    0       GP       20.00         20.00
    1       GS       20.00         20.00
    2      MIN       31.60         36.30
    3      PTS       10.20         26.50
    4       OR        1.00          1.50
    5       DR        5.30          7.20
    6      REB        6.30          8.60
    7      AST        2.80          6.50
    8      STL        0.50          1.30
    9      BLK        1.00          0.60
    10      TO        0.80          3.30
    11      PF        2.00          2.00
    12  AST/TO        3.70          2.00
    13     FGM        4.00         10.40
    14     FGA        7.40         21.90
    15     FG%       53.70         47.50
    16     3PM        2.10          2.40
    17     3PA        4.40          7.50
    18     3P%       46.60         31.50
    19     FTM        0.30          3.40
    20     FTA        0.40          4.80
    21     FT%       62.50         70.50
    22     2PM        1.90          8.10
    23     2PA        3.00         14.50
    24     2P%       64.40         55.70
    25  SC-EFF        1.39          1.21
    26  SH-EFF        0.68          0.53
    
    * Values will be different when used but the returned
      data frame should match the above structure.
    """
    # First check that the user is searching player by first and last name.
    if len(player1.split()) < 2 or len(player2.split()) < 2 :
        print("Please use first and last names for players.")
        return
    
    # Search players to get team aabreviation for searching
    # accounting for multiple search results.
    try:# Try searching for the players
        player1_df = search.player(player1)
        player2_df = search.player(player2)
    except KeyError:# Error check
        print("Please check your player search.")
    else:# Else, collect abbreviations and search stats for all players in results
        players1 = (player1_df['first_name'] + " " + player1_df['last_name']).tolist()
        teams1 = player1_df['team'].tolist()
        players2 = (player2_df['first_name'] + " " + player2_df['last_name']).tolist()
        teams2 = player2_df['team'].tolist()
        # Concatenate lists
        players_final = players1 + players2
        teams_final = teams1 + teams2
        abrv_list = []
        for team in teams_final:
            abrv_list.append(abrv_translator(search.team(team).abrv))
        # Collect stats
        stats_values = []
        metrics = []
        to_pop = []
        for i in range(0,len(abrv_list)):
            stats_r = requests.get('https://www.espn.com/nba/team/stats/_/name/' + 
                                                             abrv_list[i]).content
            stats_dict = get_espn_player(stats_r,players_final[i])
            if stats_dict != 0:
                stats_values.append(stats_dict['values'])
                metrics = stats_dict['metrics']
            else: # Handle players no longer in NBA
                to_pop.append(players_final[i])
        players_final = [e for e in players_final if e not in to_pop]
        # Make final data frame
        df_final = pd.DataFrame(data = metrics, columns = ['stat'])
        for i in range(0,len(stats_values)):
            if players_final[i] not in df_final.columns:
                df_final.insert(loc = df_final.shape[1], column = players_final[i], 
                                                value = np.round(stats_values[i],2))
        return(df_final)
    




# Import libraries
import requests
import pandas as pd
import html5lib
from nba_data_miner.helper_functions import abrv_translator
from nba_data_miner.helper_functions import get_espn_team,get_espn_player
from pandas import json_normalize

###### CLASSES #############
# Create team class
class Team():
    def __init__(self, name = '', abbreviation = '',
                 conference = '', wins = 0, losses = 0,
                 home_record = '', away_record = '',
                 players = {}):
        self.name = name
        self.abrv = abbreviation
        self.conf = conference
        self.wins = wins
        self.losses = losses
        self.home_rec = home_record
        self.away_rec = away_record
        self.players = players
    
    def __str__(self):
        players_df = pd.DataFrame.from_dict(self.players)
        return("Name: " + self.name + "\nAbbreviation: " +
               self.abrv + "\nConference: " + self.conf +
               "\nWins: " + str(self.wins) + "\nLosses: " + 
               str(self.losses) + "\nHome record: " + self.home_rec +
               "\nAway record: " + self.away_rec + "\nPlayers: \n" +
               str(players_df))
    
###### FUNCTIONS ########
def team(team = ""):
    """
    Fucntion that allows a user to perform a case insensitive
    search for a team by full name ('Brooklyn Nets'), short
    name ('Nets'), or abbreviation ('BKN').

    Parameters
    ----------
    team : String

    Returns
    -------
    A Team object defined as follows:
        Name: [string]
        Abbreviation: [string]
        Conference: [string]
        Wins: [int]
        Losses: [int]
        Home record: [string]
        Away record: [string]
        Players: [dict]

    Examples
    --------
    >>> from nba_data_miner import search
    >>> print(search.team("Celtics"))
    Name: Boston Celtics
    Abbreviation: BOS
    Conference: East
    Wins: 22
    Losses: 7
    Home record: 11-2
    Away record: 11-5
    Players:
    
                           0             1           2                 3              4        
    Name     Malcolm Brogdon  Jaylen Brown  JD Davison  Danilo Gallinari  Blake Griffin   
    POS                   PG            SG          SG                PF             PF           
    Age                   30            26          20                34             33            
    HT                 6' 4"         6' 6"       6' 1"            6' 10"          6' 9"        
    WT               229 lbs       223 lbs     195 lbs           236 lbs        250 lbs 
    College         Virginia    California     Alabama                --       Oklahoma 
    Salary       $22,600,000   $28,741,071                    $6,479,000     $1,836,090
    """

    # Request all teams from API
    try:
        r_teams = requests.get('https://www.balldontlie.io/api/v1/teams/')
    except requests.exceptions.RequestException as e:
        raise SystemExit(e)

    # Make data frame of team information
    teams_list = list(r_teams.json().values())[0]
    teams_df = pd.DataFrame.from_records(teams_list)
    
    # Search team data frame (case insensitive)
    teams_df_lower = teams_df.copy()
    teams_df_lower['abbreviation'] = teams_df_lower['abbreviation'].str.lower()
    teams_df_lower['full_name'] = teams_df_lower['full_name'].str.lower()
    teams_df_lower['name'] = teams_df_lower['name'].str.lower()
    if team.lower() not in teams_df_lower.values: # Error check argument
        raise Exception("You must search by team name or abbreviation." + 
                        " Please check function argument.")
    else:
        # Allows for search by full name, short name, and abbreviation
        if team.lower() in teams_df_lower.full_name.values:
            i = teams_df_lower.index[teams_df_lower['full_name'] == team.lower()]
        elif team.lower() in teams_df_lower.name.values:
            i = teams_df_lower.index[teams_df_lower['name'] == team.lower()]
        elif team.lower() in teams_df_lower.abbreviation.values:
            i = teams_df_lower.index[teams_df_lower['abbreviation'] == team.lower()]
        # Search teams data frame for name, abreviation, and conference
        my_name = teams_df.loc[i,'full_name'].values.item()
        my_abrv = teams_df.loc[i,'abbreviation'].values.item()
        my_conf = teams_df.loc[i,'conference'].values.item()
        
        # Scrape win loss record from ESPN 
        try:
            espn_standings = requests.get('https://www.espn.com/nba/standings/_/group/league').content
        except requests.exceptions.RequestException as e:
            raise SystemExit(e)
        espn_df_list = pd.read_html(espn_standings,flavor='html5lib')
        espn_teams = espn_df_list[0].T.reset_index().T
        espn_df = espn_df_list[1]
        espn_df.insert(loc = 0, column = "Team", value = espn_teams[0].to_list())
        for i in range(0,len(espn_df)):
            if my_name in espn_df.loc[i,'Team']:
                my_wins = espn_df.loc[i,'W']
                my_losses = espn_df.loc[i,'L']
                my_home_record = espn_df.loc[i,'HOME']
                my_away_record = espn_df.loc[i,'AWAY']
                
        # Scrape dataframe of team players and their information from ESPN
        url_abrv = abrv_translator(my_abrv)
        try:
            espn_players = requests.get('https://www.espn.com/nba/team/roster/_/name/' +
                                                                    url_abrv).content
        except requests.exceptions.RequestException as e:
            raise SystemExit(e)
        espn_players_list = pd.read_html(espn_players,flavor='html5lib')
        espn_players_df = espn_players_list[0]
        espn_players_df.drop('Unnamed: 0', inplace = True, axis = 1)
        espn_players_df['Name'] = espn_players_df['Name'].str.replace(r'\d+', '', regex = True)
        espn_players_dict = espn_players_df.to_dict('index')
        
        # Make new team object with data
        team_obj = Team(name = my_name, abbreviation = my_abrv, conference = my_conf,
                    wins = my_wins, losses = my_losses, home_record = my_home_record,
                    away_record = my_away_record, players = espn_players_dict)
        # Return
        return(team_obj)
    
def player(player):
    """
    Fucntion that allows a user to perform a case insensitive
    search for a player by full name.

    Parameters
    ----------
    player : String

    Returns
    -------
    DataFrame of the player's information 
    Type: pandas.core.frame.DataFrame

    Examples
    --------
    >>> from nba_data_miner import search
    >>> search.player("Marcus Smart")
    id first_name last_name  height_feet  height_inches position            team  weight_pounds
    0  420     Marcus     Smart            6              4        G  Boston Celtics            220
    """

    # First check that the user is searching player by first and last name.
    if len(player.split()) < 2:
        print("Please use first and last names for search.")
        return
    
    # Request player from API
    try:
        r_player = requests.get('https://www.balldontlie.io/api/v1/players/?search=' + player)
    except requests.exceptions.RequestException as e:
        raise SystemExit(e)
    player_list = list(r_player.json().values())[0]
    player_df = pd.DataFrame.from_records(player_list)
    last_name_column = player_df.pop('last_name')
    player_df.insert(2, 'last_name', last_name_column)
    for i in range(0,len(player_df)):
        team_dict = player_df.loc[i,'team']
        player_df.loc[i,'team'] = team_dict.get('full_name')
        
    return(player_df)



# Import libraries
import requests
import pandas as pd
import html5lib
from pandas import json_normalize

def abrv_translator(abrv_in):
    # Abbreviation directory to sync espn and balldontlie API
    abrv_dir = {'espn':['ATL','BOS','BKN','CHA','CHI','CLE','DAL','DEN','DET',
                       'GS','HOU','IND','LAC','LAL','MEM','MIA','MIL','MIN',
                       'NO','NY','OKC','ORL','PHI','PHX','POR','SAC','SA',
                       'TOR','UTAH','WSH'],
                'api':['ATL','BOS','BKN','CHA','CHI','CLE','DAL','DEN','DET',
                      'GSW','HOU','IND','LAC','LAL','MEM','MIA','MIL','MIN',
                      'NOP','NYK','OKC','ORL','PHI','PHX','POR','SAC','SAS',
                      'TOR','UTA','WAS']}
    abrv_df = abrv_df = pd.DataFrame(abrv_dir)
    if abrv_in not in abrv_dir['api']:
        raise Exception("That abbreviation does not match with the api." +
                        " Please run search.team(your team), to find correct abbrevation.")
    else:
        abrv_out = abrv_df.loc[abrv_df.index[abrv_df['api'] == abrv_in][0],'espn']
        return(abrv_out) # Returns espn abbreviation for searching
    
def get_espn_team(request):
    # Function to scrape stats about a team in the NBA and return a dict.
    stats_content = pd.read_html(request,flavor='html5lib')
    stats_metrics = stats_content[1].columns.tolist() + stats_content[2].columns.tolist()
    stats_values_overall = stats_content[1].iloc[len(stats_content[1])-1,:].tolist()
    stats_values_shooting =  stats_content[2].iloc[len(stats_content[2])-1,:].tolist()
    stats_values_final = stats_values_overall + stats_values_shooting

    # Check stats data frames are same length
    if len(stats_metrics) != len(stats_metrics):
            raise Exception("There seems to be a problem with the ESPN data. Check columns.")
    elif len(stats_values_final) != len(stats_values_final):
            raise Exception("There seems to be a problem with the ESPN data. Check values.")
    else:
        final_dict = {'metrics' : stats_metrics, 'values' : stats_values_final}
        return(final_dict)
    
def get_espn_player(request, player_name):
    # Function to scrape stats about a player in the NBA and return a dict.
    
    # Get index of player in team data frame
    stats_content = pd.read_html(request,flavor='html5lib')
    stats_metrics = stats_content[1].columns.tolist() + stats_content[2].columns.tolist()
    name_df = stats_content[0]
    name_count = name_df['Name'].str.contains(player_name).sum()
    if name_count < 1: # Check if player currently plays for the NBA
        return(0)
    else:
        ind = name_df[name_df['Name'].str.contains(player_name)].index.tolist()[0]
        stats_values_overall = stats_content[1].iloc[ind,:].tolist()
        stats_values_shooting =  stats_content[2].iloc[ind,:].tolist()
        stats_values_final = stats_values_overall + stats_values_shooting

        # Check stats data frames are same length
        if len(stats_metrics) != len(stats_metrics):
                raise Exception("There seems to be a problem with the ESPN data. Check columns.")
        elif len(stats_values_final) != len(stats_values_final):
                raise Exception("There seems to be a problem with the ESPN data. Check values.")
        else:
            final_dict = {'metrics' : stats_metrics, 'values' : stats_values_final}
            return(final_dict)




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nba_data_miner']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'helper-functions>=2.0.11,<3.0.0',
 'html5lib>=1.1,<2.0',
 'numpy>=1.23.5,<2.0.0',
 'pandas>=1.5.2,<2.0.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'nba-data-miner',
    'version': '0.1.0',
    'description': 'A package to help with searching and comparing players and teams in the NBA.',
    'long_description': '# nba_data_miner\n\nA package to help with searching and comparing players and teams in the NBA.\n\n## Installation\n\n```bash\n$ pip install nba_data_miner\n```\n\n## Usage\n\n- TODO\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`nba_data_miner` was created by Francis Baring. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`nba_data_miner` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Francis Baring',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/frank-baring',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
